@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import pyghmi.exceptions as exc
+import pyghmi.ipmi.private.constants as event_const
 
 
 class OEMHandler(object):
@@ -262,7 +263,7 @@ class OEMHandler(object):
         """Get graphical console launcher"""
         return ()
 
-    def add_extra_net_configuration(self, netdata):
+    def add_extra_net_configuration(self, netdata, channel=None):
         """Add additional network configuration data
 
         Given a standard netdata struct, add details as relevant from
@@ -271,14 +272,28 @@ class OEMHandler(object):
         """
         return
 
+    def get_oem_identifier(self):
+        """Get host name
+
+        """
+        return None
+
+    def set_oem_identifier(self, name):
+        """Set host name
+
+        :param name: host name to be set
+        """
+        return False
+
     def detach_remote_media(self):
         raise exc.UnsupportedFunctionality()
 
     def attach_remote_media(self, imagename, username, password):
         raise exc.UnsupportedFunctionality()
 
-    def upload_media(self, filename, progress):
-        raise exc.UnsupportedFunctionality()
+    def upload_media(self, filename, progress, data):
+        raise exc.UnsupportedFunctionality(
+            'Remote media upload not supported on this system')
 
     def list_media(self):
         raise exc.UnsupportedFunctionality()
@@ -378,8 +393,46 @@ class OEMHandler(object):
     def save_licenses(self, directory):
         raise exc.UnsupportedFunctionality()
 
-    def apply_license(self, filename, progress=None):
+    def apply_license(self, filename, progress=None, data=None):
         raise exc.UnsupportedFunctionality()
 
     def get_user_expiration(self, uid):
         return None
+
+    def set_oem_extended_privilleges(self, uid):
+        """Set user extended privillege as 'KVM & VMedia Allowed'
+
+        |KVM & VMedia Not Allowed	0x00 0x00 0x00 0x00
+        |KVM Only Allowed	0x01 0x00 0x00 0x00
+        |VMedia Only Allowed	0x02  0x00 0x00 0x00
+        |KVM & VMedia Allowed	0x03 0x00 0x00 0x00
+
+        :param uid: User ID.
+        """
+        return False
+
+    def process_zero_fru(self, zerofru):
+        return self.process_fru(zerofru)
+
+    def is_valid(self, name):
+        return name is not None
+
+    def process_password(self, password, data):
+        return data
+
+    def set_server_capping(self, value):
+        """Set power capping for server
+
+        :param value: power capping value to set.
+        """
+        pass
+
+    def get_server_capping(self):
+        """Get power capping for server
+
+        :return: power capping value.
+        """
+        return None
+
+    def get_oem_event_const(self):
+        return event_const
