@@ -25,6 +25,7 @@ import pyghmi.ipmi.oem.generic as generic
 from pyghmi.ipmi.oem.lenovo import cpu
 from pyghmi.ipmi.oem.lenovo import dimm
 from pyghmi.ipmi.oem.lenovo import drive
+from pyghmi.ipmi.oem.lenovo import energy
 from pyghmi.ipmi.oem.lenovo import firmware
 from pyghmi.ipmi.oem.lenovo import imm
 from pyghmi.ipmi.oem.lenovo import inventory
@@ -123,8 +124,44 @@ led_status = {
     0x00: "Off",
     0xFF: "On"
 }
+
+ami_leds = {
+    "BMC_HEARTBEAT": 0x00,
+    "BMC_UID": 0x01,
+    "SYSTEM_FAULT": 0x02,
+    "HDD_FAULT": 0x03
+}
+
+asrock_leds = {
+    "SYSTEM_EVENT": 0x00,
+    "BMC_UID": 0x01,
+    "LED_FAN_FAULT_1": 0x02,
+    "LED_FAN_FAULT_2": 0x03,
+    "LED_FAN_FAULT_3": 0x04
+}
+
+ts460_leds = {
+    "SYSTEM_EVENT": 0x00,
+    "BMC_UID": 0x01,
+    "LED_FAN_FAULT_1": 0x02,
+    "LED_FAN_FAULT_2": 0x03,
+    "LED_FAN_CPU": 0x04,
+    "LED_FAN_REAR": 0x05
+}
+
+asrock_led_status = {
+    0x00: "Off",
+    0x01: "On"
+}
+
+ami_led_status = {
+    0x00: "Off",
+    0x01: "On"
+}
+
 led_status_default = "Blink"
 mac_format = '{0:02x}:{1:02x}:{2:02x}:{3:02x}:{4:02x}:{5:02x}'
+categorie_items = ["cpu", "dimm", "firmware", "bios_version"]
 
 
 def _megarac_abbrev_image(name):
@@ -155,7 +192,7 @@ class OEMHandler(generic.OEMHandler):
         elif self.has_imm:
             self.immhandler = imm.IMMClient(ipmicmd)
         elif self.is_fpc:
-            self.smmhandler = nextscale.SMMClient(ipmicmd)
+            self.smmhandler = nextscale.SMMClient(ipmicmd, self.is_fpc)
         elif self.has_tsma:
             conn = wc.SecureHTTPConnection(
                 ipmicmd.bmc, 443,
@@ -165,7 +202,6 @@ class OEMHandler(generic.OEMHandler):
             self.tsmahandler.set_credentials(
                 ipmicmd.ipmi_session.userid.decode('utf-8'),
                 ipmicmd.ipmi_session.password.decode('utf-8'))
-            self.tsmahandler.init_redfish()
 
     @property
     def _megarac_eth_index(self):
@@ -285,18 +321,22 @@ class OEMHandler(generic.OEMHandler):
     def reseat_bay(self, bay):
         if self.is_fpc:
             return self.smmhandler.reseat_bay(bay)
+        elif self.has_xcc and bay == -1:
+            return self.immhandler.reseat()
         return super(OEMHandler, self).reseat_bay(bay)
 
     def get_ntp_enabled(self):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             ntpres = self.ipmicmd.xraw_command(netfn=0x32, command=0xa7)
             return ntpres['data'][0] == '\x01'
         elif self.is_fpc:
             return self.smmhandler.get_ntp_enabled(self._fpc_variant)
+        elif self.has_tsma:
+            return self.tsmahandler.get_ntp_enabled()
         return None
 
     def get_ntp_servers(self):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             srvs = []
             ntpres = self.ipmicmd.xraw_command(netfn=0x32, command=0xa7)
             srvs.append(ntpres['data'][1:129].rstrip('\x00'))
@@ -304,10 +344,12 @@ class OEMHandler(generic.OEMHandler):
             return srvs
         if self.is_fpc:
             return self.smmhandler.get_ntp_servers()
+        if self.has_tsma:
+            return self.tsmahandler.get_ntp_servers()
         return None
 
     def set_ntp_enabled(self, enabled):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             if enabled:
                 self.ipmicmd.xraw_command(
                     netfn=0x32, command=0xa8, data=(3, 1), timeout=15)
@@ -318,10 +360,12 @@ class OEMHandler(generic.OEMHandler):
         if self.is_fpc:
             self.smmhandler.set_ntp_enabled(enabled)
             return True
+        if self.has_tsma:
+            self.tsmahandler.set_ntp_enabled(enabled)
         return None
 
     def set_ntp_server(self, server, index=0):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             if not (0 <= index <= 1):
                 raise pygexc.InvalidParameterValue("Index must be 0 or 1")
             cmddata = bytearray((1 + index, ))
@@ -334,6 +378,10 @@ class OEMHandler(generic.OEMHandler):
                     'SMM supports indexes 0 through 2')
             self.smmhandler.set_ntp_server(server, index)
             return True
+        elif self.has_tsma:
+            if not (0 <= index <= 1):
+                raise pygexc.InvalidParameterValue("Index must be 0 or 1")
+            return self.tsmahandler.set_ntp_server(server, index)
         return None
 
     def set_user_access(self, uid, channel, callback, link_auth, ipmi_msg,
@@ -351,12 +399,18 @@ class OEMHandler(generic.OEMHandler):
             return self._fpc_variant
         fpc_ids = ((19046, 32, 1063), (20301, 32, 462))
         smm_id = (19046, 32, 1180)
+        smmv2_2u = (19046, 32, 1360)
+        smmv2_6u = (19046, 32, 1387)
         currid = (self.oemid['manufacturer_id'], self.oemid['device_id'],
                   self.oemid['product_id'])
         if currid in fpc_ids:
             self._fpc_variant = 6
         elif currid == smm_id:
             self._fpc_variant = 2
+        elif currid == smmv2_2u:
+            self._fpc_variant = 0x22
+        elif currid == smmv2_6u:
+            self._fpc_variant = 0x26
         return self._fpc_variant
 
     @property
@@ -366,6 +420,7 @@ class OEMHandler(generic.OEMHandler):
                                       self.oemid['product_id'])
 
     tsma_ids = ((19046, 32, 1287),)
+
     @property
     def has_tsma(self):
         currid = (self.oemid['manufacturer_id'], self.oemid['device_id'],
@@ -387,24 +442,59 @@ class OEMHandler(generic.OEMHandler):
             return True
         return False
 
+    @property
+    def has_asrock(self):
+        # True if this particular server have a ASROCKRACK
+        # based service processor (RS160 or TS460)
+        # RS160 (Riddler) product id is 1182 (049Eh)
+        # TS460 (WildThing) product id is 1184 (04A0h)
+
+        if (self.oemid['manufacturer_id'] == 19046
+                and (self.oemid['product_id'] == 1182
+                     or self.oemid['product_id'] == 1184)):
+            try:
+                self.ipmicmd.xraw_command(netfn=0x3a,
+                                          command=0x50,
+                                          data=(0x00, 0x00, 0x00))
+            except pygexc.IpmiException as ie:
+                if ie.ipmicode == 193:
+                    return False
+                raise
+            return True
+        return False
+
+    @property
+    def isTS460(self):
+        if self.oemid['product_id'] == 1184:
+            return True
+        return False
+
     def get_oem_inventory_descriptions(self):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             # Thinkserver with TSM
             if not self.oem_inventory_info:
                 self._collect_tsm_inventory()
             return iter(self.oem_inventory_info)
         elif self.has_imm:
             return self.immhandler.get_hw_descriptions()
+        elif self.is_fpc:
+            return self.smmhandler.get_inventory_descriptions(self.ipmicmd,
+                                                              self.is_fpc)
         return ()
 
     def get_oem_inventory(self):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             self._collect_tsm_inventory()
             for compname in self.oem_inventory_info:
                 yield (compname, self.oem_inventory_info[compname])
         elif self.has_imm:
             for inv in self.immhandler.get_hw_inventory():
                 yield inv
+        elif self.is_fpc:
+            for compname in self.smmhandler.get_inventory_descriptions(
+                    self.ipmicmd, self.is_fpc):
+                yield (compname, self.smmhandler.get_inventory_of_component(
+                    self.ipmicmd, compname))
 
     def get_sensor_data(self):
         if self.has_imm:
@@ -412,15 +502,21 @@ class OEMHandler(generic.OEMHandler):
                 yield self.immhandler.get_oem_sensor_reading(name,
                                                              self.ipmicmd)
         elif self.is_fpc:
-            for name in nextscale.get_sensor_names(self._fpc_variant):
+            for name in nextscale.get_sensor_names(self.ipmicmd,
+                                                   self._fpc_variant):
                 yield nextscale.get_sensor_reading(name, self.ipmicmd,
                                                    self._fpc_variant)
+        elif self.has_ami:
+            self.get_ami_sensor_data()
 
     def get_sensor_descriptions(self):
         if self.has_imm:
             return self.immhandler.get_oem_sensor_descriptions(self.ipmicmd)
         elif self.is_fpc:
-            return nextscale.get_sensor_descriptions(self._fpc_variant)
+            return nextscale.get_sensor_descriptions(
+                self.ipmicmd, self._fpc_variant)
+        elif self.has_ami:
+            self.get_ami_sensor_descriptions()
         return ()
 
     def get_sensor_reading(self, sensorname):
@@ -430,33 +526,59 @@ class OEMHandler(generic.OEMHandler):
         elif self.is_fpc:
             return nextscale.get_sensor_reading(sensorname, self.ipmicmd,
                                                 self._fpc_variant)
+        elif self.has_ami:
+            self.get_ami_sensor_reading(sensorname)
         return ()
 
     def get_inventory_of_component(self, component):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             self._collect_tsm_inventory()
             return self.oem_inventory_info.get(component, None)
         if self.has_imm:
             return self.immhandler.get_component_inventory(component)
+        if self.is_fpc:
+            return self.smmhandler.get_inventory_of_component(component)
+
+    def get_cmd_type(self, categorie_item, catspec):
+        if self.has_asrock:
+            cmd_type = catspec["command"]["asrock"]
+        elif categorie_item in categorie_items:
+            cmd_type = catspec["command"]["lenovo"]
+        else:
+            cmd_type = catspec["command"]
+
+        return cmd_type
 
     def _collect_tsm_inventory(self):
         self.oem_inventory_info = {}
+        asrock = False
+        if self.has_asrock:
+            asrock = True
         for catid, catspec in inventory.categories.items():
-            if (catspec.get("workaround_bmc_bug", False)):
+            # skip the inventory fields if the system is RS160
+            if asrock and catid not in categorie_items:
+                continue
+            if (catspec.get("workaround_bmc_bug", False)
+                    and catspec["workaround_bmc_bug"](
+                        "ami" if self.has_ami else "lenovo")):
                 rsp = None
-                tmp_command = dict(catspec["command"])
+                cmd = self.get_cmd_type(catid, catspec)
+                tmp_command = dict(cmd)
                 tmp_command["data"] = list(tmp_command["data"])
                 count = 0
                 for i in range(0x01, 0xff):
                     tmp_command["data"][-1] = i
                     try:
                         partrsp = self.ipmicmd.xraw_command(**tmp_command)
+                        count += 1
+                        if asrock and partrsp["data"][1] == "\xff":
+                            continue
+
                         if rsp is None:
                             rsp = partrsp
                             rsp["data"] = list(rsp["data"])
                         else:
                             rsp["data"].extend(partrsp["data"][1:])
-                        count += 1
                     except Exception:
                         break
                 # If we didn't get any response, assume we don't have
@@ -467,13 +589,14 @@ class OEMHandler(generic.OEMHandler):
                 rsp["data"] = buffer(bytearray(rsp["data"]))
             else:
                 try:
-                    rsp = self.ipmicmd.xraw_command(**catspec["command"])
+                    cmd = self.get_cmd_type(catid, catspec)
+                    rsp = self.ipmicmd.xraw_command(**cmd)
                 except pygexc.IpmiException:
                     continue
             # Parse the response we got
             try:
                 items = inventory.parse_inventory_category(
-                    catid, rsp,
+                    catid, rsp, asrock,
                     countable=catspec.get("countable", True)
                 )
             except Exception:
@@ -483,7 +606,22 @@ class OEMHandler(generic.OEMHandler):
 
             for item in items:
                 try:
-                    key = catspec["idstr"].format(item["index"])
+                    # Originally on ThinkServer and SD350 (Kent),
+                    # the DIMM is distinguished by slot,
+                    # the key is the value of slot number (item["index"])
+                    # While on RS160/TS460 the DIMMs is distinguished
+                    # by slot number and channel number,
+                    # the key is the value of the sum of slot number
+                    # and channel number
+                    if asrock and catid == "dimm":
+                        if item["channel_number"] == 1:
+                            key = catspec["idstr"].format(item["index"])
+                        else:
+                            key = catspec["idstr"].format(
+                                item["index"] + item["channel_number"])
+                    else:
+                        key = catspec["idstr"].format(item["index"])
+
                     del item["index"]
                     self.oem_inventory_info[key] = item
                 except Exception:
@@ -492,16 +630,42 @@ class OEMHandler(generic.OEMHandler):
                     continue
 
     def get_leds(self):
-        if self.has_tsm:
-            for (name, id_) in leds.items():
-                try:
-                    rsp = self.ipmicmd.xraw_command(netfn=0x3A, command=0x02,
+        cmd = 0x02
+        led_set = leds
+        led_set_status = led_status
+
+        asrock = self.has_asrock
+        if self.has_ami:
+            cmd = 0x05
+            led_set = ami_leds
+            led_set_status = ami_led_status
+        elif asrock:
+            cmd = 0x50
+            # because rs160 has different led info with ts460
+            if self.isTS460:
+                led_set = ts460_leds
+            else:
+                led_set = asrock_leds
+            led_set_status = asrock_led_status
+
+        for (name, id_) in led_set.items():
+            try:
+                if asrock:
+                    rsp = self.ipmicmd.xraw_command(netfn=0x3A, command=cmd,
+                                                    data=(0x03, id_, 0x00))
+                    rdata = bytearray(rsp['data'][:])
+                    status = rdata[1]
+                else:
+                    rsp = self.ipmicmd.xraw_command(netfn=0x3A, command=cmd,
                                                     data=(id_,))
-                except pygexc.IpmiException:
-                    continue  # Ignore LEDs we can't retrieve
-                status = led_status.get(ord(rsp['data'][0]),
+                    rdata = bytearray(rsp['data'][:])
+                    status = rdata[0]
+
+            except pygexc.IpmiException:
+                continue  # Ignore LEDs we can't retrieve
+            status = led_set_status.get(status,
                                         led_status_default)
-                yield (name, {'status': status})
+            yield (name, {'status': status})
 
     def set_identify(self, on, duration):
         if on and not duration and self.is_sd350:
@@ -519,13 +683,13 @@ class OEMHandler(generic.OEMHandler):
             try:
                 _, _, wwn1, wwn2, mac1, mac2 = fru['board_extra']
                 if wwn1 not in ('0000000000000000', ''):
-                    fru['WWN 1'] = wwn1
+                    fru['WWN 1'] = wwn1.encode('utf-8')
                 if wwn2 not in ('0000000000000000', ''):
-                    fru['WWN 2'] = wwn2
+                    fru['WWN 2'] = wwn2.encode('utf-8')
                 if mac1 not in ('00:00:00:00:00:00', ''):
-                    fru['MAC Address 1'] = mac1
+                    fru['MAC Address 1'] = mac1.encode('utf-8')
                 if mac2 not in ('00:00:00:00:00:00', ''):
-                    fru['MAC Address 2'] = mac2
+                    fru['MAC Address 2'] = mac2.encode('utf-8')
             except (AttributeError, KeyError):
                 pass
             try:
@@ -575,10 +739,24 @@ class OEMHandler(generic.OEMHandler):
                 pass
             if self.has_xcc and name and name.startswith('PSU '):
                 self.immhandler.augment_psu_info(fru, name)
+            if (self.has_xcc and 'memory_type' in fru
+                    and fru['memory_type'] == 'Unknown'):
+                self.immhandler.fetch_dimm(name, fru)
             return fru
-        elif self.is_fpc == 2:  # SMM variant
+        elif self.is_fpc and self.is_fpc != 6:  # SMM variant
             fru['oem_parser'] = 'lenovo'
             return self.smmhandler.process_fru(fru)
+        elif self.has_asrock:
+            fru['oem_parser'] = 'lenovo'
+            # ASRock RS160 TS460 lays out specific interpretation of the
+            # board extra fields
+            try:
+                mac1 = fru['board_extra']
+                if mac1 not in ('00:00:00:00:00:00', ''):
+                    fru['MAC Address 1'] = mac1.encode('utf-8')
+            except (AttributeError, KeyError):
+                pass
+            return fru
         else:
             fru['oem_parser'] = None
             return fru
@@ -623,19 +801,56 @@ class OEMHandler(generic.OEMHandler):
         self._hasimm = (rdata[1] & 1 == 1) or (rdata[1] & 16 == 16)
         return self._hasimm
 
+    @property
+    def has_ami(self):
+        """True if this particular server is AMI based lenovo server
+
+        """
+        if(self.oemid['manufacturer_id'] == 19046
+                and self.oemid['product_id'] == 13616):
+            try:
+                rsp = self.ipmicmd.xraw_command(netfn=0x3a, command=0x80)
+            except pygexc.IpmiException as ie:
+                if ie.ipmicode == 193:
+                    return False
+                raise
+            rdata = bytearray(rsp['data'][:])
+            if rdata[0] in range(5):
+                return True
+            else:
+                return False
+        return False
+
     def get_oem_firmware(self, bmcver, components):
-        if self.has_tsm:
+        if self.has_tsm or self.has_ami or self.has_asrock:
             command = firmware.get_categories()["firmware"]
-            rsp = self.ipmicmd.xraw_command(**command["command"])
-            return command["parser"](rsp["data"])
+            fw_cmd = self.get_cmd_type("firmware", command)
+            rsp = self.ipmicmd.xraw_command(**fw_cmd)
+
+            # the newest Lenovo ThinkServer versions are returning Bios version
+            # numbers through another command
+            bios_versions = None
+            if self.has_tsm or self.has_asrock:
+                bios_command = firmware.get_categories()["bios_version"]
+                bios_cmd = self.get_cmd_type("bios_version", bios_command)
+                bios_rsp = self.ipmicmd.xraw_command(**bios_cmd)
+                if self.has_asrock:
+                    bios_versions = bios_rsp['data']
+                else:
+                    bios_versions = bios_command["parser"](bios_rsp['data'])
+
+            # pass bios versions to firmware parser
+            return command["parser"](rsp["data"],
+                                     bios_versions,
+                                     self.has_asrock)
         elif self.has_imm:
             return self.immhandler.get_firmware_inventory(bmcver, components)
         elif self.is_fpc:
             return nextscale.get_fpc_firmware(bmcver, self.ipmicmd,
                                               self._fpc_variant)
         elif self.has_tsma:
-            return self.tsmahandler.get_firmware_inventory(components,
-                                                           raisebypass=False)
+            return self.tsmahandler.get_firmware_inventory(
+                components, raisebypass=False, ipmicmd=self.ipmicmd)
         return super(OEMHandler, self).get_oem_firmware(bmcver, components)
 
     def get_diagnostic_data(self, savefile, progress, autosuffix=False):
@@ -644,7 +859,8 @@ class OEMHandler(generic.OEMHandler):
                                                        autosuffix)
         if self.is_fpc:
             return self.smmhandler.get_diagnostic_data(savefile, progress,
-                                                       autosuffix)
+                                                       autosuffix,
+                                                       self._fpc_variant)
         if self.has_tsma:
             return self.tsmahandler.get_diagnostic_data(savefile, progress,
                                                         autosuffix)
@@ -752,6 +968,7 @@ class OEMHandler(generic.OEMHandler):
         conn.request('POST', '/rpc/WEBSES/create.asp', params, headers)
         rsp = conn.getresponse()
         if rsp.status == 200:
+            conn.cookies = {}
             body = rsp.read().split('\n')
             session_line = None
             for line in body:
@@ -784,16 +1001,19 @@ class OEMHandler(generic.OEMHandler):
                                            self.ipmicmd.ipmi_session.userid,
                                            self.ipmicmd.ipmi_session.password)
 
-    def add_extra_net_configuration(self, netdata):
+    def add_extra_net_configuration(self, netdata, channel=None):
         if self.has_tsm:
             ipv6_addr = self.ipmicmd.xraw_command(
                 netfn=0x0c, command=0x02,
                 data=(0x01, 0xc5, 0x00, 0x00))["data"][1:]
             if not ipv6_addr:
                 return
-            ipv6_prefix = ord(self.ipmicmd.xraw_command(
+            rspdata = self.ipmicmd.xraw_command(
                 netfn=0xc, command=0x02,
-                data=(0x1, 0xc6, 0, 0))['data'][1])
+                data=(0x1, 0xc6, 0, 0))['data']
+            ipv6_prefix_ba = bytearray(rspdata)
+            ipv6_prefix = ipv6_prefix_ba[1]
+
             if hasattr(socket, 'inet_ntop'):
                 ipv6str = socket.inet_ntop(socket.AF_INET6, ipv6_addr)
             else:
@@ -979,13 +1199,19 @@ class OEMHandler(generic.OEMHandler):
             return self.immhandler.get_bmc_configuration()
         if self.is_fpc:
             return self.smmhandler.get_bmc_configuration()
+        if self.has_tsma:
+            return self.tsmahandler.get_bmc_configuration()
         return super(OEMHandler, self).get_bmc_configuration()
 
     def set_bmc_configuration(self, changeset):
         if self.has_xcc:
             return self.immhandler.set_bmc_configuration(changeset)
         if self.is_fpc:
-            return self.smmhandler.set_bmc_configuration(changeset)
+            return self.smmhandler.set_bmc_configuration(
+                changeset, self._fpc_variant)
+        if self.has_tsma:
+            return self.tsmahandler.set_bmc_configuration(
+                changeset)
         return super(OEMHandler, self).set_bmc_configuration(changeset)
 
     def get_system_configuration(self, hideadvanced):
@@ -1026,10 +1252,10 @@ class OEMHandler(generic.OEMHandler):
                 netfn=0x32, command=0x9f, data=(8, 10, 0, 0))
             self.ipmicmd.xraw_command(netfn=0x32, command=0x9f, data=(8, 11))
 
-    def upload_media(self, filename, progress):
+    def upload_media(self, filename, progress, data):
         if self.has_xcc or self.has_imm:
-            return self.immhandler.upload_media(filename, progress)
-        return super(OEMHandler, self).upload_media(filename, progress)
+            return self.immhandler.upload_media(filename, progress, data)
+        return super(OEMHandler, self).upload_media(filename, progress, data)
 
     def list_media(self):
         if self.has_xcc or self.has_imm:
@@ -1063,7 +1289,93 @@ class OEMHandler(generic.OEMHandler):
             return self.immhandler.save_licenses(directory)
         return super(OEMHandler, self).save_licenses(directory)
 
-    def apply_license(self, filename, progress=None):
+    def apply_license(self, filename, progress=None, data=None):
         if self.has_xcc:
-            return self.immhandler.apply_license(filename, progress)
-        return super(OEMHandler, self).apply_license(filename, progress)
+            return self.immhandler.apply_license(filename, progress, data)
+        return super(OEMHandler, self).apply_license(filename, progress, data)
+
+    def set_oem_extended_privilleges(self, uid):
+        """Set user extended privillege as 'KVM & VMedia Allowed'
+
+        |KVM & VMedia Not Allowed	0x00 0x00 0x00 0x00
+        |KVM Only Allowed	0x01 0x00 0x00 0x00
+        |VMedia Only Allowed	0x02  0x00 0x00 0x00
+        |KVM & VMedia Allowed	0x03 0x00 0x00 0x00
+
+        :param uid: User ID.
+        """
+        if self.has_tsm:
+            self.ipmicmd.xraw_command(netfn=0x32, command=0xa3, data=(
+                uid, 0x03, 0x00, 0x00, 0x00))
+            return True
+        return False
+
+    def process_zero_fru(self, zerofru):
+        if (self.oemid['manufacturer_id'] == 19046
+                and self.oemid['product_id'] == 13616):
+            # Currently SD350 FRU UUID is synchronized with the Device UUID.
+            # Need to change to System UUID in future.
+            # Since the IPMI get device uuid matches SMBIOS,
+            # no need to decode it.
+            guiddata = self.ipmicmd.raw_command(netfn=6, command=0x8)
+            if 'error' not in guiddata:
+                zerofru['UUID'] = util.decode_wireformat_uuid(
+                    guiddata['data'], True)
+        else:
+            # It is expected that a manufacturer matches SMBIOS to IPMI
+            # get system uuid return data.  If a manufacturer does not
+            # do so, they should handle either deletion or fixup in the
+            # OEM processing pass.  Code optimistically assumes that if
+            # data is returned, than the vendor is properly using it.
+
+            guiddata = self.ipmicmd.raw_command(netfn=6, command=0x37)
+            if 'error' not in guiddata:
+                if (self.oemid['manufacturer_id'] == 19046
+                        and (self.oemid['product_id'] == 1182
+                             or self.oemid['product_id'] == 1184)):
+                    # The manufacturer (Asrockrack) of RS160/TS460
+                    # matches SMBIOS
+                    # to IPMI get system uuid return data,
+                    # no need to decode it.
+                    zerofru['UUID'] = util.decode_wireformat_uuid(
+                        guiddata['data'], True)
+                else:
+                    zerofru['UUID'] = util.decode_wireformat_uuid(
+                        guiddata['data'])
+        return self.process_fru(zerofru)
+
+    def get_ami_sensor_reading(self, sensorname):
+        """Get an OEM sensor
+
+        If software wants to model some OEM behavior as a 'sensor' without
+        doing SDR, this hook provides that ability.  It should mimic
+        the behavior of 'get_sensor_reading' in command.py.
+        """
+        for sensor in self.get_sensor_data():
+            if sensor.name == sensorname:
+                return sensor
+
+    def get_ami_sensor_descriptions(self):
+        """Get list of OEM sensor names and types
+
+        Iterate over dicts describing a label and type for OEM 'sensors'.  This
+        should mimic the behavior of the get_sensor_descriptions function
+        in command.py.
+        """
+        if self.has_ami:
+            energy_sensor = energy.Energy(self.ipmicmd)
+            for sensor in energy_sensor.get_energy_sensor():
+                yield {'name': sensor.name,
+                       'type': sensor.type}
+
+    def get_ami_sensor_data(self):
+        """Get OEM sensor data
+
+        Iterate through all OEM 'sensors' and return data as if they were
+        normal sensors.  This should mimic the behavior of the get_sensor_data
+        function in command.py.
+        """
+        if self.has_ami:
+            energy_sensor = energy.Energy(self.ipmicmd)
+            for sensor in energy_sensor.get_energy_sensor():
+                yield sensor
