@@ -15,15 +15,16 @@
 import fnmatch
 import struct
 import weakref
-from xml.etree.ElementTree import fromstring
+from xml.etree.ElementTree import fromstring as rfromstring
 import zipfile
 
+import six
+
 import pyghmi.constants as pygconst
 import pyghmi.exceptions as pygexc
 import pyghmi.ipmi.private.session as ipmisession
 from pyghmi.ipmi import sdr
 import pyghmi.util.webclient as webclient
-import six
 
 try:
     from urllib import urlencode
@@ -36,6 +37,30 @@ except NameError:
     pass
 
 
+psutypes = {
+    0x63: 'CFF v3 550W PT',
+    0x64: 'CFF v3 750W PT',
+    0x65: 'CFF v3 750W TT',
+    0x66: 'CFF v3 1100W PT',
+    0x67: 'CFF v3 1600W PT',
+    0x68: 'CFF v3 2000W PT',
+    0x6B: 'CFF v4 500W PT',
+    0x6C: 'CFF v4 750W PT',
+    0x6D: 'CFF v4 750W TT',
+    0x6E: 'CFF v4 1100W PT',
+    0x6F: 'CFF v4 1100W -48Vdc',
+    0x70: 'CFF v4 1100W 200-277Vac/240-380Vdc',
+    0x71: 'CFF v4 1800W PT',
+    0x72: 'CFF v4 2400W PT',
+}
+
+
+def fromstring(inputdata):
+    if b'!entity' in inputdata.lower():
+        raise Exception('!ENTITY not supported in this interface')
+    return rfromstring(inputdata)
+
+
 def stringtoboolean(originput, name):
     input = originput.lower()
     try:
@@ -68,25 +93,32 @@ def fpc_read_dc_output(ipmicmd):
 
 def fpc_read_fan_power(ipmicmd):
     rsp = ipmicmd.xraw_command(netfn=0x32, command=0x90, data=(3,))
-    rsp = rsp['data']
-    rsp += '\x00'
+    rsp = bytes(rsp['data'])
+    rsp += b'\x00'
     return struct.unpack_from('<I', rsp[1:])[0] / 100.0
 
 
 def fpc_read_psu_fan(ipmicmd, number, sz):
     rsp = ipmicmd.xraw_command(netfn=0x32, command=0xa5, data=(number,))
-    rsp = rsp['data']
-    return struct.unpack_from('<H', rsp[:2])[0]
+    rsp = bytes(rsp['data'])
+    if len(rsp) > 5:
+        return struct.unpack_from('<H', rsp[2:4])[0]
+    else:
+        return struct.unpack_from('<H', rsp[:2])[0]
 
 
 def fpc_get_psustatus(ipmicmd, number, sz):
     rsp = ipmicmd.xraw_command(netfn=0x32, command=0x91)
     mask = 1 << (number - 1)
+    rsp['data'] = bytearray(rsp['data'])
+    if len(rsp['data']) >= 10:
+        tmpdata = rsp['data']
+        rsp['data'] = list(struct.unpack('<HHHHBB', tmpdata[:10]))
     if len(rsp['data']) == 6:
-        statdata = bytearray([0])
+        statdata = [0]
     else:
-        statdata = bytearray()
-    statdata += bytearray(rsp['data'])
+        statdata = []
+    statdata += rsp['data']
     presence = statdata[3] & mask == mask
     pwrgood = statdata[4] & mask == mask
     throttle = (statdata[6] | statdata[2]) & mask == mask
@@ -118,11 +150,11 @@ def fpc_get_nodeperm(ipmicmd, number, sz):
     if len(rsp['data']) == 4:  # different gens handled rc differently
         rsp['data'] = b'\x00' + bytes(rsp['data'])
     elif len(rsp['data']) == 6:  # New FPC format
-        rsp['data'] = rsp['data'][:2] + rsp['data'][3:]
+        rsp['data'] = bytes(rsp['data'][:2]) + bytes(rsp['data'][3:])
     perminfo = bytearray(rsp['data'])[1]
     if sz == 6:  # FPC
         permfail = ('\x02', '\x03')
-    elif sz == 2:  # SMM
+    else:  # SMM
         permfail = ('\x02',)
     if perminfo & 0x20:
         if rsp['data'][4] in permfail:
@@ -144,6 +176,33 @@ def fpc_read_powerbank(ipmicmd):
     return struct.unpack_from('<H', rsp['data'][3:])[0]
 
 
+def get_psu_count(ipmicmd, variant):
+    if variant == 0x26:
+        mymsg = ipmicmd.xraw_command(netfn=0x32, command=0xa8)
+        builddata = bytearray(mymsg['data'])
+        if builddata[13] == 3:
+            return 9
+        else:
+            return 6
+    else:
+        return variant & 0xf
+
+
+def fpc_get_dripstatus(ipmicmd, number, sz):
+    health = pygconst.Health.Ok
+    states = []
+    rsp = ipmicmd.xraw_command(0x34, 5)
+    rdata = bytearray(rsp['data'])
+    number = number - 1
+    if rdata[0] & (1 << number) == 0:
+        states.append('Absent')
+        health = pygconst.Health.Critical
+    if rdata[1] & (1 << number) != 0:
+        states.append('Leak detected')
+        health = pygconst.Health.Critical
+    return (health, states)
+
+
 fpc_sensors = {
     'AC Power': {
         'type': 'Power',
@@ -155,6 +214,11 @@ fpc_sensors = {
         'units': 'W',
         'provider': fpc_read_dc_output,
     },
+    'PSU Power Loss': {
+        'type': 'Power',
+        'units': 'W',
+        'provider': fpc_read_dc_output,
+    },
     'Fan Power': {
         'type': 'Power',
         'units': 'W',
@@ -164,7 +228,7 @@ fpc_sensors = {
         'type': 'Fan',
         'units': 'RPM',
         'provider': fpc_read_psu_fan,
-        'elements': 1,
+        'elementsfun': get_psu_count,
     },
     'Total Power Capacity': {
         'type': 'Power',
@@ -182,34 +246,72 @@ fpc_sensors = {
         'type': 'Power Supply',
         'returns': 'tuple',
         'units': None,
+        'elements': True,
         'provider': fpc_get_psustatus,
-        'elements': 1,
+        'elementsfun': get_psu_count,
+    },
+    'Drip Sensor': {
+        'type': 'Management Subsystem Health',
+        'elements': True,
+        'abselements': 2,
+        'returns': 'tuple',
+        'units': None,
+        'provider': fpc_get_dripstatus,
     }
 }
 
 
-def get_sensor_names(size):
+def get_sensor_names(ipmicmd, size):
     global fpc_sensors
     for name in fpc_sensors:
-        if size == 2 and name in ('Fan Power', 'Total Power Capacity'):
+        if size != 6 and name in ('Fan Power', 'Total Power Capacity',
+                                  'DC Power'):
+            continue
+        if size == 6 and name == 'PSU Power Loss':
+            continue
+        if size != 0x26 and name == 'Drip Sensor':
             continue
         sensor = fpc_sensors[name]
-        if 'elements' in sensor:
-            for elemidx in range(sensor['elements'] * size):
+
+        if 'abselements' in sensor:
+            for elemidx in range(sensor['abselements']):
+                elemidx += 1
+                yield '{0} {1}'.format(name, elemidx)
+        elif 'elementsfun' in sensor:
+            for elemidx in range(sensor['elementsfun'](ipmicmd, size)):
+                elemidx += 1
+                yield '{0} {1}'.format(name, elemidx)
+        elif 'elements' in sensor:
+            for elemidx in range(sensor['elements'] * (size & 0b11111)):
                 elemidx += 1
                 yield '{0} {1}'.format(name, elemidx)
         else:
             yield name
 
 
-def get_sensor_descriptions(size):
+def get_sensor_descriptions(ipmicmd, size):
     global fpc_sensors
     for name in fpc_sensors:
-        if size == 2 and name in ('Fan Power', 'Total Power Capacity'):
+        if size != 6 and name in ('Fan Power', 'Total Power Capacity',
+                                  'DC Power'):
+            continue
+        if size == 6 and name == 'PSU Power Loss':
+            continue
+        if size != 0x26 and name == 'Drip Sensor':
             continue
         sensor = fpc_sensors[name]
-        if 'elements' in sensor:
-            for elemidx in range(sensor['elements'] * size):
+        if 'abselements' in sensor:
+            for elemidx in range(sensor['abselements']):
+                elemidx += 1
+                yield {'name': '{0} {1}'.format(name, elemidx),
+                       'type': sensor['type']}
+        elif 'elementsfun' in sensor:
+            for elemidx in range(sensor['elementsfun'](ipmicmd, size)):
+                elemidx += 1
+                yield {'name': '{0} {1}'.format(name, elemidx),
+                       'type': sensor['type']}
+        elif 'elements' in sensor:
+            for elemidx in range(sensor['elements'] * (size & 0b11111)):
                 elemidx += 1
                 yield {'name': '{0} {1}'.format(name, elemidx),
                        'type': sensor['type']}
@@ -221,10 +323,15 @@ def get_fpc_firmware(bmcver, ipmicmd, fpcorsmm):
     mymsg = ipmicmd.xraw_command(netfn=0x32, command=0xa8)
     builddata = bytearray(mymsg['data'])
     name = None
-    if fpcorsmm == 2:  # SMM
-        name = 'SMM'
-        buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
-            *[chr(x) for x in builddata[-7:]])
+    if fpcorsmm != 6:  # SMM
+        if fpcorsmm >> 5:
+            name = 'SMM2'
+            buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
+                *[chr(x) for x in builddata[6:13]])
+        else:
+            name = 'SMM'
+            buildid = '{0}{1}{2}{3}{4}{5}{6}'.format(
+                *[chr(x) for x in builddata[5:12]])
     elif len(builddata) == 8:
         builddata = builddata[1:]  # discard the 'completion code'
         name = 'FPC'
@@ -232,7 +339,12 @@ def get_fpc_firmware(bmcver, ipmicmd, fpcorsmm):
     bmcmajor, bmcminor = [int(x) for x in bmcver.split('.')]
     bmcver = '{0}.{1:02d}'.format(bmcmajor, bmcminor)
     yield (name, {'version': bmcver, 'build': buildid})
-    yield ('PSOC', {'version': '{0}.{1}'.format(builddata[2], builddata[3])})
+    if fpcorsmm == 6:
+        yield ('PSOC', {'version': '{0}.{1}'.format(builddata[2],
+                                                    builddata[3])})
+    else:
+        yield ('PSOC', {'version': '{0}.{1}'.format(builddata[3],
+                                                    builddata[4])})
 
 
 def get_sensor_reading(name, ipmicmd, sz):
@@ -246,12 +358,20 @@ def get_sensor_reading(name, ipmicmd, sz):
     else:
         bnam, _, idx = name.rpartition(' ')
         idx = int(idx)
-        if bnam in fpc_sensors and idx <= fpc_sensors[bnam]['elements'] * sz:
-            sensor = fpc_sensors[bnam]
-            if 'returns' in sensor:
-                health, states = sensor['provider'](ipmicmd, idx, sz)
-            else:
-                value = sensor['provider'](ipmicmd, idx, sz)
+        if bnam in fpc_sensors:
+            max = -1
+            if 'abselements' in fpc_sensors[bnam]:
+                max = fpc_sensors[bnam]['abselements']
+            elif 'elementsfun' in fpc_sensors[bnam]:
+                max = 99
+            elif 'elements' in fpc_sensors[bnam]:
+                max = fpc_sensors[bnam]['elements'] * sz
+            if idx <= max:
+                sensor = fpc_sensors[bnam]
+                if 'returns' in sensor:
+                    health, states = sensor['provider'](ipmicmd, idx, sz)
+                else:
+                    value = sensor['provider'](ipmicmd, idx, sz)
     if sensor is not None:
         return sdr.SensorReading({'name': name, 'imprecision': None,
                                   'value': value, 'states': states,
@@ -263,7 +383,8 @@ def get_sensor_reading(name, ipmicmd, sz):
 
 class SMMClient(object):
 
-    def __init__(self, ipmicmd):
+    def __init__(self, ipmicmd, variant):
+        self.smm_variant = variant
         self.ipmicmd = weakref.proxy(ipmicmd)
         self.smm = ipmicmd.bmc
         self.username = ipmicmd.ipmi_session.userid
@@ -303,30 +424,39 @@ class SMMClient(object):
         rspbody = rsp.read()
         accountinfo = fromstring(rspbody)
         for rule in self.rulemap:
-            settings[rule] = {'value': int(
-                accountinfo.find(self.rulemap[rule]).text)}
-        rsp = self.ipmicmd.xraw_command(0x34, 3)
-        fanmode = self.fanmodes[bytearray(rsp['data'])[0]]
-        settings['fanspeed'] = {
-            'value': fanmode, 'default': 'Normal',
-            'help': ('Adjust the fan speed of the D2 Chassis. Capped settings '
-                     'will reduce fan speed for better acoustic experience at '
-                     'the expense of performance.  Normal is using the Lenovo '
-                     'engineered cooling adjustments across the full range. '
-                     'Boosted adds fanspeed to the Normal response to '
-                     'provide more aggressive cooling.'),
-            'possible': [self.fanmodes[x] for x in self.fanmodes]}
+            ruleinfo = accountinfo.find(self.rulemap[rule])
+            if ruleinfo is not None:
+                settings[rule] = {'value': int(ruleinfo.text)}
+        dwc = self.ipmicmd.xraw_command(0x32, 0x94)
+        dwc = bytearray(dwc['data'])
+        if len(dwc) not in (3, 4) or dwc[0] == 1:
+            rsp = self.ipmicmd.xraw_command(0x34, 3)
+            fanmode = self.fanmodes[bytearray(rsp['data'])[0]]
+            settings['fanspeed'] = {
+                'value': fanmode, 'default': 'Normal',
+                'help': ('Adjust the fan speed of the D2 Chassis. Capped '
+                         'settings will reduce fan speed for better acoustic '
+                         'experience at the expense of performance.  Normal '
+                         'is using the Lenovo engineered cooling adjustments '
+                         'across the full range. Boosted adds fanspeed to '
+                         'the Normal response to provide more aggressive '
+                         'cooling.'),
+                'possible': [self.fanmodes[x] for x in self.fanmodes]}
         powercfg = self.ipmicmd.xraw_command(0x32, 0xa2)
         powercfg = bytearray(powercfg['data'])
+        if len(powercfg) == 5:
+            powercfg = powercfg[1:]
         val = powercfg[0]
-        if val == 1:
+        if val == 2:
+            val = 'N+N'
+        elif val == 1:
             val = 'N+1'
         elif val == 0:
             val = 'Disable'
         settings['power_redundancy'] = {
             'default': 'N+1',
             'value': val,
-            'possible': ['N+1', 'Disable'],
+            'possible': ['N+N', 'N+1', 'Disable'],
             'help': ('Configures allowed power budget according to expected '
                      'redundancy. If N+1, power caps will be set to keep '
                      'servers from using more power than the installed power '
@@ -378,7 +508,7 @@ class SMMClient(object):
             pass
         return settings
 
-    def set_bmc_configuration(self, changeset):
+    def set_bmc_configuration(self, changeset, variant):
         rules = []
         powercfg = [None, None]
         sendhost = None
@@ -393,7 +523,9 @@ class SMMClient(object):
                     rules.append('{0}:{1}'.format(
                         self.rulemap[rule], changeset[key]['value']))
             if fnmatch.fnmatch('power_redundancy', key.lower()):
-                if 'n+1'.startswith(changeset[key]['value'].lower()):
+                if 'n+n'.startswith(changeset[key]['value'].lower()):
+                    powercfg[0] = 2
+                elif 'n+1'.startswith(changeset[key]['value'].lower()):
                     powercfg[0] = 1
                 elif 'disable'.startswith(changeset[key]['value'].lower()):
                     powercfg[0] = 0
@@ -429,14 +561,20 @@ class SMMClient(object):
             self.wc.request('POST', '/data', rules)
             self.wc.getresponse().read()
         if powercfg != [None, None]:
-            if None in powercfg:
-                currcfg = self.ipmicmd.xraw_command(0x32, 0xa2)
-                currcfg = bytearray(currcfg['data'])
-                if powercfg[0] is None:
-                    powercfg[0] = currcfg[0]
-                if powercfg[1] is None:
-                    powercfg[1] = currcfg[1]
-            self.ipmicmd.xraw_command(0x32, 0xa3, data=powercfg)
+            if variant != 6:
+                if None in powercfg:
+                    currcfg = self.ipmicmd.xraw_command(0x32, 0xa2)
+                    currcfg = bytearray(currcfg['data'])
+                    if powercfg[0] is None:
+                        powercfg[0] = currcfg[0]
+                    if powercfg[1] is None:
+                        powercfg[1] = currcfg[1]
+                self.ipmicmd.xraw_command(0x32, 0xa3, data=powercfg)
+            elif variant == 6:
+                if powercfg[0] is not None:
+                    self.ipmicmd.xraw_command(0x32, 0xa3, data=powercfg[:1])
+                if powercfg[1] is not None:
+                    self.ipmicmd.xraw_command(0x32, 0x9c, data=powercfg[1:])
         if sendhost is not None:
             sendhost = 1 if sendhost else 0
             self.ipmicmd.xraw_command(0xc, 1, data=[1, 0xc5, sendhost])
@@ -448,8 +586,10 @@ class SMMClient(object):
         if priv.lower() == 'administrator':
             rsp = self.ipmicmd.xraw_command(netfn=6, command=0x46, data=(uid,))
             username = bytes(rsp['data']).rstrip(b'\x00')
+            if not isinstance(username, str):
+                username = username.decode('utf8')
             self.wc.request(
-                'POST', '/data', b'set=user({0},1,{1},511,,4,15,0)'.format(
+                'POST', '/data', 'set=user({0},1,{1},511,,4,15,0)'.format(
                     uid, username))
             rsp = self.wc.getresponse()
             rsp.read()
@@ -458,7 +598,8 @@ class SMMClient(object):
         self.ipmicmd.xraw_command(netfn=0x32, command=0xa4,
                                   data=[int(bay), 2])
 
-    def get_diagnostic_data(self, savefile, progress=None):
+    def get_diagnostic_data(self, savefile, progress=None, autosuffix=False,
+                            variant=None):
         rsp = self.ipmicmd.xraw_command(netfn=0x32, command=0xb1, data=[0])
         if bytearray(rsp['data'])[0] != 0:
             raise Exception("Service data generation already in progress")
@@ -479,7 +620,12 @@ class SMMClient(object):
                 progress({'phase': 'initializing', 'progress': initpct})
         if self.wc is None:
             raise Exception("Failed to connect to web api")
-        url = '/preview/smm-ffdc.tgz?ST1={0}'.format(self.st1)
+        if variant and variant >> 5:
+            url = '/preview/smm2-ffdc.tgz?ST1={0}'.format(self.st1)
+        else:
+            url = '/preview/smm-ffdc.tgz?ST1={0}'.format(self.st1)
+        if autosuffix and not savefile.endswith('.tgz'):
+            savefile += '-smm-ffdc.tgz'
         fd = webclient.FileDownloader(self.wc, url, savefile)
         fd.start()
         while fd.isAlive():
@@ -492,13 +638,17 @@ class SMMClient(object):
         return savefile
 
     def process_fru(self, fru):
+        smmv1 = self.smm_variant & 0xf0 == 0
         # TODO(jjohnson2): can also get EIOM, SMM, and riser data if warranted
-        fru['Serial Number'] = self.ipmicmd.xraw_command(
-            netfn=0x32, command=0xb0, data=(5, 1))['data'][:].strip(
-                ' \x00\xff').replace('\xff', '')
-        fru['Model'] = self.ipmicmd.xraw_command(
-            netfn=0x32, command=0xb0, data=(5, 0))['data'][:].strip(
-                ' \x00\xff').replace('\xff', '')
+        snum = bytes(self.ipmicmd.xraw_command(
+            netfn=0x32, command=0xb0, data=(5, 1))['data'][:])
+        mnum = bytes(self.ipmicmd.xraw_command(
+            netfn=0x32, command=0xb0, data=(5, 0))['data'][:])
+        if not smmv1:
+            snum = snum[2:]
+            mnum = mnum[2:]
+        fru['Serial Number'] = snum.strip(b' \x00\xff').replace(b'\xff', b'')
+        fru['Model'] = mnum.strip(b' \x00\xff').replace(b'\xff', b'')
         return fru
 
     def get_webclient(self):
@@ -537,6 +687,8 @@ class SMMClient(object):
             if rsp.status != 200:
                 raise Exception(rsp.read())
             indexhtml = rsp.read()
+            if not isinstance(indexhtml, str):
+                indexhtml = indexhtml.decode('utf8')
             for line in indexhtml.split('\n'):
                 if '"ST1"' in line:
                     self.st1 = line.split()[-1].replace(
@@ -550,6 +702,8 @@ class SMMClient(object):
             body = rsp.read()
             if rsp.status != 200:
                 raise Exception(body)
+            if not isinstance(body, str):
+                body = body.decode('utf8')
             for line in body.split('\n'):
                 if '"ST1"' in line:
                     self.st1 = line.split()[-1].replace(
@@ -616,6 +770,8 @@ class SMMClient(object):
             1 if enabled else 0))
         rsp = self.wc.getresponse()
         result = rsp.read()
+        if not isinstance(result, str):
+            result = result.decode('utf8')
         self.logout()
         if '<status>ok</status>' not in result:
             raise Exception("Unrecognized result: " + result)
@@ -625,6 +781,8 @@ class SMMClient(object):
             index + 1, server))
         rsp = self.wc.getresponse()
         result = rsp.read()
+        if not isinstance(result, str):
+            result = result.decode('utf8')
         if '<status>ok</status>' not in result:
             raise Exception("Unrecognized result: " + result)
         self.logout()
@@ -648,8 +806,15 @@ class SMMClient(object):
     def update_firmware(self, filename, data=None, progress=None, bank=None):
         if progress is None:
             progress = lambda x: True
-        if not data and zipfile.is_zipfile(filename):
+        z = None
+        if data and hasattr(data, 'read'):
+            if zipfile.is_zipfile(data):
+                z = zipfile.ZipFile(data)
+            else:
+                data.seek(0)
+        elif data is None and zipfile.is_zipfile(filename):
             z = zipfile.ZipFile(filename)
+        if z:
             for tmpname in z.namelist():
                 if tmpname.endswith('.rom'):
                     filename = tmpname
@@ -700,6 +865,48 @@ class SMMClient(object):
             complete = percent >= 100.0
         return 'complete'
 
+    def get_inventory_descriptions(self, ipmicmd, variant):
+        psucount = get_psu_count(ipmicmd, variant)
+        for idx in range(psucount):
+            yield 'PSU {}'.format(idx + 1)
+
+    def get_inventory_of_component(self, ipmicmd, component):
+        psuidx = int(component.replace('PSU ', ''))
+        return self.get_psu_info(ipmicmd, psuidx)
+
+    def get_psu_info(self, ipmicmd, psunum):
+        psuinfo = ipmicmd.xraw_command(0x34, 0x6, data=(psunum,))
+        psuinfo = bytearray(psuinfo['data'])
+        psutype = struct.unpack('<H', psuinfo[23:25])[0]
+        psui = {}
+        if psuinfo[0] != 1:
+            return {'Model': 'Unavailable'}
+        psui['Revision'] = psuinfo[34]
+        psui['Description'] = psutypes.get(
+            psutype, 'Unknown ({})'.format(psutype))
+        psui['Part Number'] = str(psuinfo[35:47].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['FRU Number'] = str(psuinfo[47:59].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Serial Number'] = str(psuinfo[59:71].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Header Code'] = str(psuinfo[71:75].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Vendor'] = str(psuinfo[25:29].strip(
+            b' \x00\xff').decode('utf8'))
+        psui['Manufacturing Date'] = '20{}-W{}'.format(
+            psuinfo[77:79].decode('utf8'), psuinfo[75:77].decode('utf8'))
+        psui['Primary Firmware Version'] = '{:x}.{:x}'.format(
+            psuinfo[80], psuinfo[79])
+        psui['Secondary Firmware Version'] = '{:x}.{:x}'.format(
+            psuinfo[82], psuinfo[81])
+        psui['Model'] = str(psuinfo[5:23].strip(b' \x00\xff').decode('utf8'))
+        psui['Manufacturer Location'] = str(
+            psuinfo[83:85].strip(b' \x00\xff').decode('utf8'))
+        psui['Barcode'] = str(psuinfo[85:108].strip(
+            b' \x00\xff').decode('utf8'))
+        return psui
+
     def logout(self):
         self.wc.request('POST', '/data/logout', None)
         rsp = self.wc.getresponse()
