@@ -25,6 +25,7 @@ with discontinuous values for a field that was until that point
 possible to derive in a formulaic way
 """
 
+import math
 import struct
 
 jedec_ids = [
@@ -721,7 +722,7 @@ class SPD(object):
             fineoffset = 0 - ((fineoffset ^ 0xff) + 1)
         fineoffset = (finetime * fineoffset) * 10 ** -3
         mtb = spd[10] / float(spd[11])
-        clock = 2 // ((mtb * spd[12] + fineoffset) * 10 ** -3)
+        clock = math.floor(2 / ((mtb * spd[12] + fineoffset) * 10 ** -3))
         self.info['speed'] = speed_from_clock(clock)
         self.info['ecc'] = (spd[8] & 0b11000) != 0
         self.info['module_type'] = module_types.get(spd[3] & 0xf, 'Unknown')
@@ -735,8 +736,8 @@ class SPD(object):
         self.info['manufacture_date'] = decode_spd_date(spd[120], spd[121])
         self.info['serial'] = hex(struct.unpack(
             '>I', struct.pack('4B', *spd[122:126]))[0])[2:].rjust(8, '0')
-        self.info['model'] = struct.pack('18B', *spd[128:146]).strip(
-            '\x00\xff ')
+        self.info['model'] = struct.pack('20B', *spd[128:148]).strip(
+            b'\x00\xff ')
 
     def _decode_ddr4(self):
         spd = self.rawdata
@@ -744,7 +745,8 @@ class SPD(object):
             fineoffset = spd[125]
             if fineoffset & 0b10000000:
                 fineoffset = 0 - ((fineoffset ^ 0xff) + 1)
-            clock = 2 // ((0.125 * spd[18] + fineoffset * 0.001) * 0.001)
+            clock = math.floor(
+                2 / ((0.125 * spd[18] + fineoffset * 0.001) * 0.001))
             self.info['speed'] = speed_from_clock(clock)
         else:
             self.info['speed'] = 'Unknown'
@@ -755,11 +757,13 @@ class SPD(object):
         buswidth = ddr3_bus_width[spd[13] & 0b111]
         sdramwidth = ddr3_dev_width[spd[12] & 0b111]
         ranks = ddr3_ranks[(spd[12] & 0b111000) >> 3]
+        if spd[6] & 0b11 == 0b10:
+            ranks = ranks * (((spd[6] >> 4) & 0b111) + 1)
         self.info['capacity_mb'] = sdramcap / 8 * buswidth / sdramwidth * ranks
         self.info['manufacturer'] = decode_manufacturer(spd[320], spd[321])
         self.info['manufacture_location'] = spd[322]
         self.info['manufacture_date'] = decode_spd_date(spd[323], spd[324])
         self.info['serial'] = hex(struct.unpack(
             '>I', struct.pack('4B', *spd[325:329]))[0])[2:].rjust(8, '0')
-        self.info['model'] = struct.pack('18B', *spd[329:347]).strip(
+        self.info['model'] = struct.pack('20B', *spd[329:349]).strip(
             b'\x00\xff ')
