@@ -36,10 +36,10 @@ import string
 import struct
 import weakref
 
+import six
+
 import pyghmi.constants as const
 import pyghmi.exceptions as exc
-import pyghmi.ipmi.private.constants as ipmiconst
-import six
 
 try:
     import cPickle as pickle
@@ -53,6 +53,30 @@ TYPE_FRU = 2
 shared_sdrs = {}
 
 
+oem_type_offsets = {
+    343: {  # Intel
+        149: {  # Cascade Lake-AP
+            0x7a: {
+                0xda: {
+                    3: {
+                        'desc': 'Allowed',
+                        'severity': const.Health.Ok,
+                    },
+                    4: {
+                        'desc': 'Restricted',
+                        'severity': const.Health.Ok,
+                    },
+                    5: {
+                        'desc': 'Disabled',
+                        'severity': const.Health.Ok,
+                    },
+                },
+            },
+        },
+    },
+}
+
+
 def ones_complement(value, bits):
     # utility function to help with the large amount of 2s
     # complement prevalent in ipmi spec
@@ -263,12 +287,15 @@ class SDREntry(object):
     external code to pay attention to this class.
     """
 
-    def __init__(self, entrybytes, ipmicmd, reportunsupported=False):
+    def __init__(self, entrybytes, event_consts, reportunsupported=False,
+                 mfg_id=0, prod_id=0):
+        self.mfg_id = mfg_id
+        self.prod_id = prod_id
+        self.event_consts = event_consts
         # ignore record id for now, we only care about the sensor number for
         # moment
         self.readable = True
         self.reportunsupported = reportunsupported
-        self.ipmicmd = ipmicmd
         if entrybytes[2] != 0x51:
             # only recognize '1.5', the only version defined at time of writing
             raise NotImplementedError
@@ -349,9 +376,10 @@ class SDREntry(object):
         # event only, compact and full are very similar
         # this function handles the common aspects of compact and full
         # offsets from spec, minus 6
+        self.sensor_owner = entry[0]
         self.sensor_lun = entry[1] & 0x03
         self.sensor_number = entry[2]
-        self.entity = ipmiconst.entity_ids.get(
+        self.entity = self.event_consts.entity_ids.get(
             entry[3], 'Unknown entity {0}'.format(entry[3]))
         if self.rectype == 3:
             self.sensor_type_number = entry[5]
@@ -360,7 +388,7 @@ class SDREntry(object):
             self.sensor_type_number = entry[7]
             self.reading_type = entry[8]  # table 42-1
         try:
-            self.sensor_type = ipmiconst.sensor_type_codes[
+            self.sensor_type = self.event_consts.sensor_type_codes[
                 self.sensor_type_number]
         except KeyError:
             self.sensor_type = "UNKNOWN type " + str(self.sensor_type_number)
@@ -392,6 +420,8 @@ class SDREntry(object):
             # or hyphen, so go with space
             self.unit_mod = " "
         self.percent = ''
+        # two bytes of assertion event mask / lower threshold reading mask
+        self.assertion_reading_mask = entry[9:11]
         if entry[15] & 1 == 1:
             self.percent = '% '
         if self.sensor_type_number == 0xb:
@@ -426,15 +456,20 @@ class SDREntry(object):
             self.decode_formula(entry[19:25])
 
     def _decode_state(self, state):
-        mapping = ipmiconst.generic_type_offsets
+        mapping = self.event_consts.generic_type_offsets
         try:
             if self.reading_type in mapping:
                 desc = mapping[self.reading_type][state]['desc']
                 health = mapping[self.reading_type][state]['severity']
             elif self.reading_type == 0x6f:
-                mapping = ipmiconst.sensor_type_offsets
+                mapping = self.event_consts.sensor_type_offsets
                 desc = mapping[self.sensor_type_number][state]['desc']
                 health = mapping[self.sensor_type_number][state]['severity']
+            elif self.reading_type >= 0x70 and self.reading_type <= 0x7f:
+                sensedata = oem_type_offsets[self.mfg_id][self.prod_id][
+                    self.reading_type][self.sensor_type_number][state]
+                desc = sensedata['desc']
+                health = sensedata['severity']
             else:
                 desc = "Unknown state %d" % state
                 health = const.Health.Warning
@@ -444,7 +479,7 @@ class SDREntry(object):
             health = const.Health.Warning
         return desc, health
 
-    def decode_sensor_reading(self, reading):
+    def decode_sensor_reading(self, ipmicmd, reading):
         numeric = None
         output = {
             'name': self.sensor_name,
@@ -464,8 +499,8 @@ class SDREntry(object):
         if numeric is not None:
             lowerbound = numeric - (0.5 + (self.tolerance / 2.0))
             upperbound = numeric + (0.5 + (self.tolerance / 2.0))
-            lowerbound = self.decode_value(lowerbound)
-            upperbound = self.decode_value(upperbound)
+            lowerbound = self.decode_value(ipmicmd, lowerbound)
+            upperbound = self.decode_value(ipmicmd, upperbound)
             output['value'] = (lowerbound + upperbound) / 2.0
             output['imprecision'] = output['value'] - lowerbound
             discrete = False
@@ -481,53 +516,55 @@ class SDREntry(object):
         output['health'] = const.Health.Ok
         if discrete:
             for state in range(8):
-                if reading[2] & (0b1 << state):
+                if reading[2] & (0b1 << state) & \
+                        self.assertion_reading_mask[0]:
                     statedesc, health = self._decode_state(state)
                     output['health'] |= health
                     output['states'].append(statedesc)
                     output['state_ids'].append(self.assert_trap_value(state))
             if len(reading) > 3:
                 for state in range(7):
-                    if reading[3] & (0b1 << state):
+                    if reading[3] & (0b1 << state) & \
+                            self.assertion_reading_mask[0]:
                         statedesc, health = self._decode_state(state + 8)
                         output['health'] |= health
                         output['states'].append(statedesc)
                         output['state_ids'].append(
                             self.assert_trap_value(state + 8))
         else:
-            if reading[2] & 0b1:
+            if reading[2] & 0b1 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Warning
                 output['states'].append(lower + " non-critical threshold")
                 output['state_ids'].append(self.assert_trap_value(1))
-            if reading[2] & 0b10:
+            if reading[2] & 0b10 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Critical
                 output['states'].append(lower + " critical threshold")
                 output['state_ids'].append(self.assert_trap_value(2))
-            if reading[2] & 0b100:
+            if reading[2] & 0b100 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Failed
                 output['states'].append(lower + " non-recoverable threshold")
                 output['state_ids'].append(self.assert_trap_value(3))
-            if reading[2] & 0b1000:
+            if reading[2] & 0b1000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Warning
                 output['states'].append(upper + " non-critical threshold")
                 output['state_ids'].append(self.assert_trap_value(4))
-            if reading[2] & 0b10000:
+            if reading[2] & 0b10000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Critical
                 output['states'].append(upper + " critical threshold")
                 output['state_ids'].append(self.assert_trap_value(5))
-            if reading[2] & 0b100000:
+            if reading[2] & 0b100000 & self.assertion_reading_mask[0]:
                 output['health'] |= const.Health.Failed
                 output['states'].append(upper + " non-recoverable threshold")
                 output['state_ids'].append(self.assert_trap_value(6))
         return SensorReading(output, self.unit_suffix)
 
-    def _set_tmp_formula(self, value):
-        rsp = self.ipmicmd.raw_command(netfn=4, command=0x23,
-                                       data=(self.sensor_number, value))
+    def _set_tmp_formula(self, ipmicmd, value):
+        rsp = ipmicmd.raw_command(netfn=4, command=0x23,
+                                  data=(self.sensor_number, value))
         # skip next reading field, not used in on-demand situation
         self.decode_formula(rsp['data'][1:])
 
-    def decode_value(self, value):
+    def decode_value(self, ipmicmd, value):
         # Take the input value and return meaningful value
         linearization = self.linearization
         if linearization > 11:  # direct calling code to get factors
@@ -537,7 +574,7 @@ class SDREntry(object):
             # fashion.  However for now opt for retrieving rows as needed
             # rather than tracking all that information for a relatively
             # rare behavior
-            self._set_tmp_formula(value)
+            self._set_tmp_formula(ipmicmd, value)
             linearization = 0
         # time to compute the pre-linearization value.
         decoded = float((value * self.m + self.b)
@@ -741,7 +778,7 @@ class SDR(object):
                     if size == 0xff:  # get just 5 to get header to know length
                         size = 5
                     elif size > 5:
-                        size /= 2
+                        size //= 2
                         # push things over such that it's less
                         # likely to be just 1 short of a read
                         # and incur a whole new request
@@ -799,10 +836,15 @@ class SDR(object):
             if self.sensors[number].readable:
                 yield number
 
+    def make_sdr_entry(self, sdrbytes):
+        return SDREntry(sdrbytes, self.ipmicmd.get_event_constants(),
+                        False, self.mfg_id, self.prod_id)
+
     def add_sdr(self, sdrbytes):
-        newent = SDREntry(sdrbytes, self.ipmicmd)
+        newent = self.make_sdr_entry(sdrbytes)
         if newent.sdrtype == TYPE_SENSOR:
-            id = '{0}.{1}'.format(newent.sensor_number, newent.sensor_lun)
+            id = '{0}.{1}.{2}'.format(
+                newent.sensor_owner, newent.sensor_number, newent.sensor_lun)
             if id in self.sensors:
                 self.broken_sensor_ids[id] = True
                 return
