@@ -51,22 +51,6 @@ powerstates = {
     'boot': None,
 }
 
-boot_devices_write = {
-    'net': 'Pxe',
-    'network': 'Pxe',
-    'pxe': 'Pxe',
-    'hd': 'Hdd',
-    'usb': 'Usb',
-    'cd': 'Cd',
-    'cdrom': 'Cd',
-    'optical': 'Cd',
-    'dvd': 'Cd',
-    'floppy': 'Floppy',
-    'default': 'None',
-    'setup': 'BiosSetup',
-    'bios': 'BiosSetup',
-    'f1': 'BiosSetup',
-}
 
 boot_devices_read = {
     'BiosSetup': 'setup',
@@ -254,6 +238,14 @@ class Command(object):
         self._varfwinventory = None
         self._oem = None
         self._gpool = pool
+        self._bmcv4ip = None
+        for addrinf in socket.getaddrinfo(bmc, 0, 0, socket.SOCK_STREAM):
+            if addrinf[0] == socket.AF_INET:
+                self._bmcv4ip = socket.inet_pton(addrinf[0], addrinf[-1][0])
+            elif addrinf[0] == socket.AF_INET6:
+                theip = addrinf[-1][0]
+                theip = theip.split('%')[0]
+                self._bmcv6ip = socket.inet_pton(addrinf[0], theip)
         self.wc.set_header('Accept', 'application/json')
         self.wc.set_header('User-Agent', 'pyghmi')
         self.wc.set_header('Accept-Encoding', 'gzip')
@@ -676,31 +668,7 @@ class Command(object):
         :raises: PyghmiException on an error.
         :returns: dict or True -- If callback is not provided, the response
         """
-        reqbootdev = bootdev
-        if (bootdev not in boot_devices_write
-                and bootdev not in boot_devices_read):
-            raise exc.InvalidParameterValue('Unsupported device %s'
-                                            % repr(bootdev))
-        bootdev = boot_devices_write.get(bootdev, bootdev)
-        if bootdev == 'None':
-            payload = {'Boot': {'BootSourceOverrideEnabled': 'Disabled'}}
-        else:
-            payload = {'Boot': {
-                'BootSourceOverrideEnabled': 'Continuous' if persist
-                                             else 'Once',
-                'BootSourceOverrideTarget': bootdev,
-            }}
-            if uefiboot is not None:
-                uefiboot = 'UEFI' if uefiboot else 'Legacy'
-                payload['BootSourceOverrideMode'] = uefiboot
-                try:
-                    self._do_web_request(self.sysurl, payload, method='PATCH')
-                    return {'bootdev': reqbootdev}
-                except Exception:
-                    del payload['BootSourceOverrideMode']
-        thetag = self.sysinfo.get('@odata.etag', None)
-        self._do_web_request(self.sysurl, payload, method='PATCH', etag=thetag)
-        return {'bootdev': reqbootdev}
+        return self.oem.set_bootdev(bootdev, persist, uefiboot, self)
 
     @property
     def _biosurl(self):
@@ -807,6 +775,16 @@ class Command(object):
             if not nicinfo.get('InterfaceEnabled', True):
                 # skip disabled interfaces
                 continue
+            for addrs in nicinfo.get('IPv4Addresses', []):
+                v4addr = socket.inet_pton(
+                    socket.AF_INET, addrs.get('Address', '0.0.0.0'))
+                if self._bmcv4ip == v4addr:
+                    return curl
+            for addrs in nicinfo.get('IPv6Addresses', []):
+                v6addr = socket.inet_pton(
+                    socket.AF_INET6, addrs.get('Address', '::'))
+                if self._bmcv6ip == v6addr:
+                    return curl
             foundnics += 1
             lastnicurl = curl
         if name is None and foundnics != 1:
@@ -1091,7 +1069,8 @@ class Command(object):
                 self._set_system_configuration(changeset)
                 return
             except exc.RedfishError as re:
-                if 'etag' not in re.msgid.lower():
+                if ('etag' not in re.msgid.lower()
+                        and 'PreconditionFailed' not in re.msgid):
                     raise
 
     def _set_system_configuration(self, changeset):
@@ -1139,7 +1118,8 @@ class Command(object):
                 if not normval.endswith('*'):
                     normval += '*'
                 for cand in currsettings[change]['possible']:
-                    if fnmatch(cand.lower(), normval):
+                    if fnmatch(cand.lower().replace(' ', ''),
+                               normval.replace(' ', '')):
                         changeset[change] = cand
                         break
                 else:
@@ -1515,14 +1495,21 @@ class Command(object):
             vminfo = self._do_web_request(vmurl, cache=False)
             if vminfo.get('ConnectedVia', None) != 'NotConnected':
                 continue
-            try:
-                self._do_web_request(vmurl, {'Image': url, 'Inserted': True},
-                                     'PATCH')
-            except exc.RedfishError as re:
-                if re.msgid.endswith(u'PropertyUnknown'):
-                    self._do_web_request(vmurl, {'Image': url}, 'PATCH')
-                else:
-                    raise
+            inserturl = vminfo.get(
+                'Actions', {}).get(
+                    '#VirtualMedia.InsertMedia', {}).get('target', None)
+            if inserturl:
+                self._do_web_request(inserturl, {'Image': url})
+            else:
+                try:
+                    self._do_web_request(vmurl,
+                                         {'Image': url, 'Inserted': True},
+                                         'PATCH')
+                except exc.RedfishError as re:
+                    if re.msgid.endswith(u'PropertyUnknown'):
+                        self._do_web_request(vmurl, {'Image': url}, 'PATCH')
+                    else:
+                        raise
             break
 
     def detach_remote_media(self):
@@ -1538,17 +1525,23 @@ class Command(object):
             for vminfo in self._do_bulk_requests(vmurls):
                 vminfo, currl = vminfo
                 if vminfo['Image']:
-                    try:
-                        self._do_web_request(currl,
-                                             {'Image': None,
-                                              'Inserted': False},
-                                             method='PATCH')
-                    except exc.RedfishError as re:
-                        if re.msgid.endswith(u'PropertyUnknown'):
-                            self._do_web_request(currl, {'Image': None},
+                    ejurl = vminfo.get(
+                        'Actions', {}).get(
+                            '#VirtualMedia.EjectMedia', {}).get('target', None)
+                    if ejurl:
+                        self._do_web_request(ejurl, {})
+                    else:
+                        try:
+                            self._do_web_request(currl,
+                                                 {'Image': None,
+                                                  'Inserted': False},
                                                  method='PATCH')
-                        else:
-                            raise
+                        except exc.RedfishError as re:
+                            if re.msgid.endswith(u'PropertyUnknown'):
+                                self._do_web_request(currl, {'Image': None},
+                                                     method='PATCH')
+                            else:
+                                raise
 
     def upload_media(self, filename, progress=None, data=None):
         """Upload a file to be hosted on the target BMC
